﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections.Generic;
using SimpleJSON;
using System.Text;

namespace MacGruber
{
    public class BreathingBenchmark : MVRScript
    {
		private Breathing myBreathing;
		private bool myHasHijacked = false;
		
		JSONStorableString mySequenceString;
		private List<SequenceEntry> mySequence = new List<SequenceEntry>();
		private int myIndex = -1;
		private string mySequenceData = "";

        public override void Init()
		{			
			myBreathing = Utils.FindWithinSamePlugin<Breathing>(this);
			if (myBreathing == null)
				return;
			
			myBreathing.myRhythmRandomness.val = 0.0f;
			
			Utils.OnInitUI(CreateUIElement);
			Utils.SetupInfoTextNoScroll(this,
				"<color=#606060><size=40><b>BreathingBenchmark</b></size>\nSpecialized plugin to eliminate most random decisions from the Life Breathing system.\n\nThis is NOT intended to be used in context of normal scenes. No support will be given.</color>",
				350.0f, false
			);
						
			mySequenceString = new JSONStorableString("BreathSequence", "", OnSetBreathSequence);
			mySequenceString.isStorable = mySequenceString.isRestorable = false;
			RegisterString(mySequenceString);
			
			Utils.SetupAction(this, "ChooseBreath", OnChooseBreath);
			Utils.SetupAction(this, "ResetBreath", OnResetBreath);
		}
		
		private void Update()
		{
			if (!myHasHijacked && myBreathing != null && myBreathing.DebugIsInitialized())
			{
				myIndex = -1;
				OnSetBreathSequence(mySequenceData);
				myBreathing.DebugReset();
				myHasHijacked = true;
			}
		}
		
		private void OnDestroy()
		{
			Utils.OnDestroyUI();
		}
				
		private void OnSetBreathSequence(string v)
		{
			mySequenceString.valNoCallback = string.Empty;
			mySequence.Clear();
			myIndex = -1;
			mySequenceData = v;
			
			int start = 0;
			int split = -1;
			string token = "\r\n";
			do
			{
				split = mySequenceData.IndexOf(token, start);
				int end = split >= 0 ? split : mySequenceData.Length;
				string breathName = mySequenceData.Substring(start, end-start);				
				foreach (var item in myBreathing.Datasets)
				{
					Breathing.Dataset dataset = item.Value;
					int index = dataset.myEntries.FindIndex((Breathing.BreathEntry be) => { return be.Name == breathName; });
					if (index >= 0)
					{
						mySequence.Add(new SequenceEntry(dataset, index));
						break;
					}
				}				
				start = split + token.Length;
			} while (split >= 0 && start < mySequenceData.Length);
		}
		
		private void OnChooseBreath()
		{
			if (mySequence.Count == 0 || myBreathing == null)
				return;
			
			if (myBreathing.NeedNextQueueDebugEntry())
				++myIndex;
			if (myIndex < 0 || myIndex >= mySequence.Count)
				myIndex = 0;
			myBreathing.QueueDebugEntry(mySequence[myIndex]);
		}
		
		private void OnResetBreath()
		{
			if (myBreathing == null || !myHasHijacked)
				return;
			
			myIndex = -1;
			myBreathing.DebugReset();
		}
		
		public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
		{
			JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
			if (includePhysical || forceStore)
			{
				needsStore = true;
				jc["Sequence"] = mySequenceData;
			}
			return jc;
		}

		public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
		{
			base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
			if (!base.physicalLocked && restorePhysical)
			{
				mySequenceData = jc["Sequence"];
				if (mySequenceData == null)
					mySequenceData = "";
				myHasHijacked = false;
			}
		}
	}
	
	internal struct SequenceEntry
	{
		public Breathing.Dataset myDataset;
		public int myIndex;
		
		public SequenceEntry(Breathing.Dataset dataset, int index)
		{
			myDataset = dataset;
			myIndex = index;
		}
	}
	
	// Force override needed to let the Life plugin find its package
	public static partial class Utils
    {
		static Utils()
		{
			PackagePathOverride = "MacGruber.Life.13:/";  // <= Make sure to refer to the correct package here!
			PluginPathOverride = PackagePathOverride + "Custom/Scripts/MacGruber/Life/";
			IsInPackageOverride = true;
		}
	}
	
	public partial class Breathing : MVRScript
	{
		////////////////////////////////////////////////////////////////////////////////////
		// Debug Extensions

		internal Dictionary<string, Dataset> Datasets { get { return myDatasets; } }
		
		internal bool DebugIsInitialized()
		{
			return myInitialized;
		}
		
		internal void DebugReset()
		{
			// cleanup
			myQueue.Clear();
			mySelection.Clear();
			myDurationDamped = -1;
			myMultiBreathIndex = 0;
			
			// stop audio
			myHeadAudioSource.StopAndClearQueue();
			
			// reset morphs
			DriverBreathing driver = Utils.FindWithinSamePlugin<DriverBreathing>(this);
			driver.enabledJSON.val = false;
			driver.enabledJSON.val = true;
			
			// setup queue			
			myQueue.Enqueue(new Breath());
			myQueue.Enqueue(new Breath() { Duration = 1.0f });
			myQueue.Enqueue(new Breath());
			UpdateBreathTime();
			Choose();
			Breath currentBreath = Get(CURR);
			CurrentTime = currentBreath.Start;
			VelocityClampScale = currentBreath.VCS;
			myQueueInitialized = true;
			for (int i=0; i<myEvents.Count; ++i)
				myEvents[i].Reset();
		}
		
		internal bool NeedNextQueueDebugEntry()
		{
			return myQueueType != TYPE_DEBUG;
		}

		internal void QueueDebugEntry(SequenceEntry aEntry)
		{
			myQueueType = TYPE_DEBUG;
			myQueueDebugDataset = aEntry.myDataset.myName;
			myQueueDebugEntry = aEntry.myIndex;
		}
	}
}
